SLM={event={}}
SLM.NAME_MAX_LENGTH = 20 --max number of characters for preset name, limited by button size
SLM.NOTES = {
	main = "562368870", --spells LOADOUTS 0
	import = "562368871", --1 for Import --TODO
	export = "562368873", --3 for Export --TODO
	bak = "562368875", --5 for Save
}
--GOAL command line color
SLM.version = '1.0.4'

--## Choco ##--

--List of names not allowed for saved loadouts
SLM.RESERVED_NAMES = {['ACTIVE SHIP']=true,UNASSIGNED=true,['NO SHIP']=true,['-NO LOADOUTS FOUND-']=true,['BASE SHIP']=true,}
local invalid_names = {} --list of loadout names that need to be changed (queue for multiple calls from update function)

SLM.data = {}
local data = {} --{presets={},stations={}} --saved to system notes
local loadouts_index = {} --which stations use preset, reconstructed based on notes data

local vars
dofile'choco.lua'
SLM.vars, vars = dofile'vars.lua'
SLM.ui = dofile'ui.lua'

function SLM.BEGIN_DATA() end

--Returns true: loaded data from notes, false: no data found, nil: error encountered
function SLM.LoadData(notesid) --SLM.LoadData
	notesid = tonumber(notesid) or SLM.NOTES.main
	
	data = unspickle(LoadSystemNotes(notesid) or "") or {}
	local isData = next(data)~=nil
	data.stations = data.stations or {}
	data.presets = data.presets or {}
	
	--create presets index table from notes data
	for stationid,presets in pairs(data.stations) do
		for i,name in pairs(presets) do --i=1,4
			if name then
				loadouts_index[name] = loadouts_index[name] or {}
				loadouts_index[name][stationid] = loadouts_index[name][stationid] or {}
				loadouts_index[name][stationid][i] = true
			end
		end
	end
	
	--SLM.replace_station_titles() --NOT redundant, sometimes event occurs after LoadShipPresets() --OBSOLETE
	
	return isData
end

--Save data to specified system notesid (for backups) or use default
function SLM.SaveData(notesid) --SLM.SaveData
	notesid = tonumber(notesid) or SLM.NOTES.main
	
	if next(data) then --data is loaded
		data.version = SLM.version
		data.timestamp = os.time()
		SaveSystemNotes(spickle(data), notesid)
		
		return true
	else return false end --no data to save
end

function SLM.data.LoadData() if not next(data) then SLM.LoadData() end end --Load data if not already loaded

function SLM.data.GetLoadouts() SLM.data.LoadData(); return data.presets end --Returns table of all saved loadouts

--Returns loadout with specified name if it exists, or nil
function SLM.data.GetLoadoutByName(name)
	SLM.data.LoadData()
	return data.presets[name]
end

--Returns table of up to 4 loadouts saved at specified stationid, or returns loadout in slot i if i specified, nil of none
function SLM.data.GetLoadoutByStation(stationid, i)
	SLM.data.LoadData()
	
	i = tonumber(i)
	stationid = stationid or GetStationLocation()
	
	if i and i >=1 and i <=4 then
		return data.stations[stationid] and data.stations[stationid][i]
	else return data.stations[stationid] end
end

--returns table listing stationids where loadout with name is saved, or nil
function SLM.data.GetStationsByLoadout(name)
	SLM.data.LoadData()
	
	return loadouts_index[name]
end

--remove loadout i from stationid (does not save config)
local function clearStationIndex(stationid, i)
	local name = SLM.data.GetLoadoutByStation(stationid, i) --name of preset to remove
	
	if name then		
		--remove entry from station loadouts
		data.stations[stationid][i] = nil
		if not next(data.stations[stationid]) then data.stations[stationid] = nil end --remove table if empty
		
		--remove entry from names index table
		if loadouts_index[name] and loadouts_index[name][stationid] then
			loadouts_index[name][stationid][i] = nil --remove preset from station
			if not next(loadouts_index[name][stationid]) then
				loadouts_index[name][stationid] = nil --remove station entry from list if has no saved presets
				if not next(loadouts_index[name]) then loadouts_index[name] = nil end --remove entry for name if not used in any other stations
			end
		end
		
		return true
	else return false end		
end

--Overwrite default preset i with loadout
local function doDefaultAssignment(loadout, i)
	i = tonumber(i)
	if type(loadout)~='table' or not i then return false end
	
	LoadShipPresets() --restore defaults from config
	ShipPresets[i] = loadout --replace default
	SaveShipPresets() --save defaults to config
end

--assigns preset to station -- parameters must be validated before executing this function
local function doStationAssignment(name, stationid, i)
	SLM.data.LoadData()
	
	if not name then return end
	
	clearStationIndex(stationid, i) --remove old loadout in slot i
			
	--link name of loadout with station
	data.stations[stationid] = data.stations[stationid] or {}
	data.stations[stationid][i] = name			
	
	--save entry in names index table
	loadouts_index[name] = loadouts_index[name] or {}
	loadouts_index[name][stationid] = loadouts_index[name][stationid] or {}
	loadouts_index[name][stationid][i] = true
	
	--GOAL only if necessary
	SLM.replace_station_titles()
end

--replace default preset in config file with one specified or else use the one from station
local function replacePreset(i, loadout)
	local stationid = GetStationLocation()
	local name = SLM.data.GetLoadoutByStation(stationid, i)
	local station_loadout = SLM.data.GetLoadoutByName(name)
	
	loadout = loadout or station_loadout --if loadout not specified use loadout for the current station
	
	if loadout then
		ShipPresets[i] = loadout
		SaveShipPresets(i) --overwrite default loadout in config
		return true
	else return false end
end

--original actions for buy preset buttons (default presets)
local default_actions = {
	StationEquipmentBuyShipTab[1][3][6][1][1][1].action,
	StationEquipmentBuyShipTab[1][3][6][1][1][2].action,
	StationEquipmentBuyShipTab[1][3][6][1][2][1].action,
	StationEquipmentBuyShipTab[1][3][6][1][2][2].action,
}
--IUP.buttons for buy preset buttons
local default_buttons = {
	StationEquipmentBuyShipTab[1][3][6][1][1][1],
	StationEquipmentBuyShipTab[1][3][6][1][1][2],
	StationEquipmentBuyShipTab[1][3][6][1][2][1],
	StationEquipmentBuyShipTab[1][3][6][1][2][2],
}
--original actions for set presets buttons
local preset_actions = {
	StationEquipmentManageShipSelectionTab[3][5][1][2][1].action,
	StationEquipmentManageShipSelectionTab[3][5][1][2][2].action,
	StationEquipmentManageShipSelectionTab[3][5][1][3][1].action,
	StationEquipmentManageShipSelectionTab[3][5][1][3][2].action,
}
--IUP.buttons for set presets buttons
local preset_buttons = {
	StationEquipmentManageShipSelectionTab[3][5][1][2][1],
	StationEquipmentManageShipSelectionTab[3][5][1][2][2],
	StationEquipmentManageShipSelectionTab[3][5][1][3][1],
	StationEquipmentManageShipSelectionTab[3][5][1][3][2],
}
--original actions for buyback buttons


--Override LoadShipPresets function to use custom buyback button titles
local load_presets_old = LoadShipPresets
function LoadShipPresets(...)
	local val = load_presets_old(...) --call original function
	SLM.replace_station_titles() --override buttons
	
	return val
end

--Override SaveShipPresets function to use custom buyback button titles
local save_presets_old = SaveShipPresets
function SaveShipPresets(...)
	local val = save_presets_old(...) --call original function
	--SLM.replace_station_titles() --override buttons
	
	return val
end

local buyback_dlg = BuybackQuestionPrompt
local buyback_default_buttons = {} --original buttons on buyback dlg
local buyback_default_actions = {} --actions for original buyback buttons
local buyback_preset_actions = {} --actions for new buyback buttons
local buyback_queue = {BuybackQuestionPrompt}

--Override MakeBuyBackQuestionDlg function
local make_buyback_old = MakeBuyBackQuestionDlg
function MakeBuyBackQuestionDlg(...)
	local dlg = make_buyback_old(...)
	
	SLM.data.LoadData()
	
	--save reference to button actions
	buyback_default_actions = {
		dlg[1][1][1][3][1].action,
		dlg[1][1][1][3][2].action,
		dlg[1][1][1][3][3].action,
		dlg[1][1][1][3][4].action,
	}
	
	--save reference to buttons
	buyback_default_buttons = {
		dlg[1][1][1][3][1],
		dlg[1][1][1][3][2],
		dlg[1][1][1][3][3],
		dlg[1][1][1][3][4],
	}
	
	buyback_dlg = dlg
	
	--assign function for new buttons
	for i=1,4 do
		buyback_preset_actions[i] = function(...)
			local default_loadout = ShipPresets[i] --save reference to default loadout
			local isOverride = replacePreset(i) --override default loadout in config file with station's loadout
			local val = buyback_default_actions[i](...) --make purchase using overriden loadout
			if isOverride then replacePreset(i, default_loadout) end --restore default loadout in config file
			
			HideDialog(dlg)
			return val
		end
	end
	
	return dlg
end

--New action to perform when clicking buyback and set preset buttons
for i=1,4 do
	default_buttons[i].action = function(...)
		local default_loadout = ShipPresets[i] --save reference to default loadout
		local isOverride = replacePreset(i) --override default loadout in config file with station's loadout
		local val = default_actions[i](...) --make purchase using overriden loadout
		if isOverride then replacePreset(i, default_loadout) end --restore default loadout in config file
		
		return val
	end
	
	preset_buttons[i].action = function(self)
		local set_preset_cb = function(name)
			if name~="DEFAULT" then
				SLM.save(self, {i, name})
			else preset_actions[i](self) end --save as default (original button function)
		end
		
		local rename_dlg = SLM.ui.MakeRenameDialog(set_preset_cb)
		ShowDialog(rename_dlg,iup.CENTER,iup.CENTER)
	end
end

--for case where player reloads interface while in station w/o active ship
function SLM.event:PLAYER_ENTERED_GAME(event, data)
	UnregisterEvent(self, "PLAYER_ENTERED_GAME")
	
	--hack to ensure function is called after LoadShipPresets() is called externally
    local init_cb = function()
    	if buyback_dlg and buyback_dlg.visible=="YES" then
    		SLM.replace_buyback_titles()
    	end
    end
    
    SLM.init_timer = Timer()
    SLM.init_timer:SetTimeout(600, init_cb)
end

--Override buyback button titles when entering station
function SLM.event:ENTERED_STATION(event, data)	
	--LoadShipPresets()
	SLM.replace_buyback_titles() --must call before replacing station titles (which recreates buyback dialog)
	SLM.replace_station_titles()
end

--Replace text on set preset buttons in current station
function SLM.replace_station_titles(titles, is_use_color)
	--local use_color = is_use_color or (titles==nil and true or false) --only use color when station presets are used or is_use_color flag set
	titles = titles or {}
	
	if PlayerInStation() and not GetStationType() then --not in capship
		local stationid = GetStationLocation()
		local station_presets = SLM.data.GetLoadoutByStation(stationid) or {}
		
		load_presets_old() --load default presets first to clear existing text
		
		for i=1,4 do
			titles[i] = titles[i] or station_presets[i]
			titles[i] = titles[i] and titles[i]:len() > SLM.NAME_MAX_LENGTH and titles[i]:sub(1,SLM.NAME_MAX_LENGTH-3)..'...' or titles[i] --truncate
			
			if titles[i] then
				--calculate color
				local color = SLM.GetLoadoutColor(titles[i] and SLM.data.GetLoadoutByName(titles[i])) or ''
				default_buttons[i].title = color..titles[i]
			end
		end
		
		--iup.Refresh(StationEquipmentBuyShipTab) --neccessary?
	end
end

--Append station preset buttons to buyback dialog
function SLM.replace_buyback_titles(titles, is_use_color)
	titles = titles or {}
	
	if not buyback_dlg then --[[console_print'buyback empty';]] return end --DEBUG
	
	--local stationid = (tonumber(GetHomeStation()) or 0)*256+1 --hack to get stationid from sectorid --OBSOLETE
	local stationid = GetStationLocation ()
	local station_presets = SLM.data.GetLoadoutByStation(stationid) or {}
	
	--if not next(station_presets) then console_print'System Notes not yet loaded'; return end --DEBUG
	
	local new_buttons = {}
	for i=1,4 do
		titles[i] = titles[i] or station_presets[i]
		titles[i] = titles[i] and titles[i]:len() > SLM.NAME_MAX_LENGTH and titles[i]:sub(1,SLM.NAME_MAX_LENGTH-3)..'...' or titles[i] --truncate
		
		if titles[i] then
			--calculate color
			local color = SLM.GetLoadoutColor(station_presets[i] and data.presets[ station_presets[i] ]) or ''		
			
			new_buttons[i] = iup.stationbutton{title=color..titles[i]}
			new_buttons[i].action=buyback_preset_actions[i]
		else new_buttons[i] = iup.fill{size="100"} end
	end
	
	new_buttons.gap=6
	local new_button_box = iup.hbox(new_buttons)
	
	--remove existing buttons if exist
	if buyback_dlg[1][1][1][4] then iup.Destroy(buyback_dlg[1][1][1][4]) end
	
	iup.Append(buyback_dlg[1][1][1], new_button_box)
	buyback_dlg[1][1][1][4] = new_button_box
	
	--iup.Map(iup.GetDialog(new_button_box))
	iup.Map(buyback_dlg)
    iup.Refresh(buyback_dlg)
end

--returns escaped string with ship color from loadout
function SLM.GetLoadoutColor(loadout)
	if SLM.vars.GetVars("IsUseColor") and loadout and loadout.shipcolor then
		local rgb = ShipPalette_string[loadout.shipcolor+1] --index shift: ShipPallette is 1-256, not 0-255
		return rgbtohex(rgb)
	else return nil end
end

--verify name (string) is a valid preset name. If invalid return false and the error message, return true if valid
--if isSkipUnique is true then don't verify name is unique (to allow verification of existing names)
function SLM.VerifyName(name, isSkipUnique)	
	if not name or name=='' then return false, "Name must contain at least one letter" end
	if tonumber(name) then return false, "Name must contain at least one non-numeric char" end
	if name:len() > 20 then return false, "Name cannot be longer than 20 characters" end
	if name:match'^DEFAULT' then return false, "Name cannot begin with DEFAULT" end
	if name:match'^[%*%#]' then return false, "Name cannot begin with * or #" end
	if name:match'[,:]' then return false, "Name cannot contain , or :" end
	if SLM.RESERVED_NAMES[name] then return false, "That name is reserved and cannot be used" end
	if SLM.data.GetLoadoutByName(name) and not isSkipUnique then return false, "There is already a loadout with that name" end
	
	return true
end

--Hack to truncate names of ships to fit set default preset buttons
function SLM.GetTruncatedShipName(loadout)
	if type(loadout)~='table' then return end
	
	local ship_name = InvManager.GetInventoryItemParameter(loadout.ship, 'Name')
	ship_name = ship_name and ship_name:len() > SLM.NAME_MAX_LENGTH-2 and ship_name:sub(1,SLM.NAME_MAX_LENGTH-5)..'...' or ship_name --truncate
	
	return ship_name
end

--Check if specified loadout is available at current station
--return 1: price of ship + addons, nil if ship unavailable
--return 2: false if not all addons available, 'STORAGE' if some from storage, 'BUY' if all can be bought
--return 3: table of itemtypeids of addons and their availability
function SLM.CheckAvailability(loadout)
	local total_price = 0
	local is_avail = 'BUY' --assume true until proven otherwise
	local addon_availability = {}
	local stationid = PlayerInStation() and GetStationLocation()
	
	if type(loadout)~='table' then return false end --loadout invalid
	if not stationid then return false end --must be docked in station
	
	--create list of items sold by station
	local purchase_itemtypeids = {} --table with itemtypeids used as keys for all ships and addons sold by station
	local item_types = {battery=true,lightweapon=true,heavyweapon=true,turretweapon=true,ship=true} --valid item types, exclude commodities
	for i=1,GetNumStationMerch() do
		local item = GetStationMerchInfo(i)
		
		if item_types[item.type] and item.usable then
			--verify license levels
			local meets_license = true --assume true until proven otherwise
			for i,level in ipairs(item.neededlevels or {}) do
				if (tonumber(level) or 0) > GetLicenseLevel(i) then meets_license = false end
			end
			
			if  meets_license and item.itemid then purchase_itemtypeids[item.itemid]=(item.price or 0) end
		end
	end
	
	--local storage_ships=GetStationShipList(stationid) --list of ships in storage: Don't care about ships in storage!!
	local storage_addons=GetStationAddonList(stationid) --list of addons in storage
	
	local storage_itemtypeids = {} --table with itemtypeids used as keys for all ships and addons in storage
	--for _,shipid in ipairs(storage_ships) do storage_itemtypeids[GetInventoryItemType(shipid)]=true end
	for _,itemid in ipairs(storage_addons) do storage_itemtypeids[GetInventoryItemType(itemid)]=true end
	
	total_price = purchase_itemtypeids[loadout.ship] --start with ship purchase price (nil if unavailable)
	addon_availability[loadout.ship] = purchase_itemtypeids[loadout.ship] and 'BUY' --set ship availability
	
	for i,itemtypeid in ipairs(loadout.ports) do
		if itemtypeid>0 then --exclude empty ports
			if purchase_itemtypeids[itemtypeid] then --addon available for purchase
				total_price = total_price and total_price + purchase_itemtypeids[itemtypeid]
				addon_availability[itemtypeid] = 'BUY'
			elseif storage_itemtypeids[itemtypeid] then --addon in storage
				is_avail = is_avail and 'STORAGE'
				addon_availability[itemtypeid] = 'STORAGE'
			else is_avail = false end
		end
	end
	
	return total_price, is_avail, addon_availability
end

--Optimized check availability function for sorting. Does not return data table
function SLM.QuickAvailability(loadout)
	local total_price = 0
	local is_avail = 'BUY' --assume true until proven otherwise
	local stationid = PlayerInStation() and GetStationLocation()
	
	if type(loadout)~='table' then return false end --loadout invalid
	if not stationid then return false end --must be docked in station
	
	--create list of items sold by station
	local purchase_itemtypeids = {} --table with itemtypeids used as keys for all ships and addons sold by station
	local item_types = {battery=true,lightweapon=true,heavyweapon=true,turretweapon=true,ship=true} --valid item types, exclude commodities
	for i=1,GetNumStationMerch() do
		local item = GetStationMerchInfo(i)
		
		if item_types[item.type] and item.usable then
			--verify license levels
			local meets_license = true --assume true until proven otherwise
			for i,level in ipairs(item.neededlevels or {}) do
				if (tonumber(level) or 0) > GetLicenseLevel(i) then meets_license = false end
			end
			
			if  meets_license and item.itemid then purchase_itemtypeids[item.itemid]=(item.price or 0) end
		end
	end
	
	local storage_addons=GetStationAddonList(stationid) --list of addons in storage
	
	local storage_itemtypeids = {} --table with itemtypeids used as keys for all ships and addons in storage
	for _,itemid in ipairs(storage_addons) do storage_itemtypeids[GetInventoryItemType(itemid)]=true end
	
	if purchase_itemtypeids[loadout.ship] then --ship available for purchase
		total_price = purchase_itemtypeids[loadout.ship] --start with ship purchase price
	else return false end --ship not available
	
	for i,itemtypeid in ipairs(loadout.ports) do
		if itemtypeid>0 then --exclude empty ports
			if purchase_itemtypeids[itemtypeid] then --addon available for purchase
				total_price = total_price and total_price + purchase_itemtypeids[itemtypeid]
			elseif storage_itemtypeids[itemtypeid] then --addon in storage
				is_avail = is_avail and 'STORAGE'
			else return false end
		end
	end
	
	return total_price, is_avail
end

--Process console commands -- redirect to appropriate function
SLM.inputs = {save=true,assign=true,remove=true,delete=true,list=true,buy=true,rename=true,help=true,update=true,setdoubleclickdelay=true,debug=false} --TODO add export/import
function SLM:input(args)
	args = args or {}
	local command = args[1] and args[1]:lower() or nil--'help'
	
	if not GetCurrentSectorid() then OpenAlarm("", "Please login to use SLM", "OK"); return end --verify player is logged in
	SLM.data.LoadData() --load data if not alredy loaded
	
	if SLM.inputs[command] then
		table.remove(args, 1)
		SLM[command](self, args)
	else SLM.ui:Show() end --Open GUI by default
end

--saves current ship to specified loadout slot at this station
function SLM:save(args)
	--TODO use selected ship for preset buttons instead of current ship
	if args and args[1] then
		local is_default,i = string.match(args[1], '^(%u*)%s?(%d+)') --extract number from "DEFAULT N"
		i = tonumber(i)
		is_default = is_default=="DEFAULT"
		local name = args[2] --new name for loadout
		local loadout = args[3] --table, not specified from console
		local stationid = PlayerInStation() and GetStationLocation()
		
		--Safety checks
		if not (i and i>=0 and i<=4) then print'\127ffffffSlot number must be 0-4'; return end
		if math.floor(i) ~= i then print'\127ffffffSlot number must be an integer'; return end
		if not name and not (is_default and loadout) then print'\127ffffffYou must specify a name of the loadout to save at this station'; return end
		if type(loadout)~='nil' and type(loadout)~='table' then print'\127ffffffInvalid loadout'; return end
		if i>0 and (not stationid or GetStationType()) then print'\127ffffffYou must dock at a station first'; return end
		local is_valid_name, error_msg = SLM.VerifyName(name)
		if not is_valid_name then print('\127ffffff'..error_msg); return end
		
		if not SLM.data.GetLoadoutByName(name) then
			
			local is_preset --Check to determine if save function called from PDA set preset buttons
			for i,button in ipairs(preset_buttons) do if self==button then is_preset=i; break end end
			
			--determine loadout to save
			if is_preset then --use selected ship
				local default_loadout = ShipPresets[i] --save reference to default loadout
				local val = preset_actions[i](self) --save selected loadout to config
				loadout = ShipPresets[i] --read loadout that was saved to config file
				replacePreset(i, default_loadout) --restore default loadout in config file
			else loadout = loadout or SaveShipLoadout(GetActiveShipID()) end
			
			if not is_default then
				data.presets[name] = loadout --save loadout in data
				if i>0 then doStationAssignment(name, stationid, i) end --slot 0 saves new loadout but does not assign to the station
				
				SLM.SaveData()
			else doDefaultAssignment(loadout, i) end
			
			print('\127ffffffSLM: '..(i>0 and 'Loadout for slot '..i..' saved' or 'Created loadout '..name))
			return true
		else print('\127ffffff'..name..' has already been assigned. To reuse '..name..' at this station do: /SLM assign '..i..' '..name); return end
	else print'\127ffffffYou must specify a slot (0-4) to save the current loadout to: e.g. /SSL save 1'; return end
end

--assigns previously saved loadout to this station
function SLM:assign(args)
	if args and args[1] then
		local is_default,i = string.match(args[1], '^(%u*)%s?(%d+)')
		i = tonumber(i)
		is_default = is_default=="DEFAULT"
		local name = args[2] --existing name of loadout to assign
		local loadout = args[3] --table, not specified from console
		local stationid = PlayerInStation() and GetStationLocation()
		
		loadout = loadout or SLM.data.GetLoadoutByName(name)
		
		--safety checks
		if not (i and i>=1 and i<=4) then print'\127ffffffSlot number must be 1-4'; return end
		if math.floor(i) ~= i then print'\127ffffffSlot number must be an integer'; return end
		if not name and not (is_default and loadout) then print'\127ffffffYou must specify the name of the loadout to assign at this station'; return end
		if type(loadout)~='nil' and type(loadout)~='table' then print'\127ffffffInvalid loadout'; return end
		if not stationid or GetStationType() then print'\127ffffffYou must dock at a station first'; return end
		
		if loadout then
			if not is_default then
				doStationAssignment(name, stationid, i)
				SLM.SaveData()
			else doDefaultAssignment(loadout, i) end
			
			print('\127ffffffSLM: Loadout for slot '..i..' saved')
			return true
		else print('\127ffffffLoadout does not exist: '..name) end
	else print'\127ffffffYou must specify a slot (1-4) to save the specified loadout to: e.g. /SSL assign 1 NAME' end
end

--removes loadout for specified slot from current station
function SLM:remove(args)
	if args and args[1] then
		local i = tonumber(args[1])
		local stationid = PlayerInStation() and GetStationLocation()
		
		if not (i>=1 and i<=4) then print'\127ffffffSlot number must be 1-4'; return end
		if math.floor(i) ~= i then print'\127ffffffSlot number must be an integer'; return end
		if not stationid or GetStationType() then print'\127ffffffYou must dock at a station first'; return end
		
		if clearStationIndex(stationid, i) then
			SLM.SaveData()
			LoadShipPresets()
			print('\127ffffffSLM: Loadout for slot '..i..' removed')
			
			return true
		else print('\127ffffffNo loadout data found for slot '..i) end
	else print'\127ffffffYou must specify a slot (1-4) to remove: e.g. /SSL remove 1' end
end

--deletes specified loadout and removes from all stations
function SLM:delete(args)
	local name = args and args[1]
	
	if not name then print'\127ffffffYou must specify the name of the loadout to delete'; return end
	if string.len(name) > SLM.NAME_MAX_LENGTH then print('\127ffffffName must not be longer than '..SLM.NAME_MAX_LENGTH..' characters'); return end
	if not SLM.data.GetLoadoutByName(name) then print('\127ffffffloadout '..name..' not found'); return end
	
	--remove all instances of loadout:name in data.stations
	if loadouts_index[name] then --only if loadout is used as station preset
		for stationid,station_presets in pairs(loadouts_index[name]) do
			for i,_ in pairs(station_presets) do
				if data.stations[stationid][i]==name then data.stations[stationid][i]=nil end --verify preset is one to remove and remove it
			end
				
			if table.maxn(data.stations[stationid])==0 then data.stations[stationid]=nil end --remove station entry if all presets removed
		end
	end
	
	loadouts_index[name] = nil
	data.presets[name] = nil --delete preset data
	
	SLM.SaveData()
	
	--GOAL only if necessary
	SLM.replace_station_titles()
	
	print('\127ffffffSLM: Loadout '..name..' deleted')
	return true
end

--list loadouts to chat
function SLM:list(args)
	--TODO list filters
	--TODO multiple arguments
	args = args or {}
	local command = args[1] and args[1]:lower()
	
	if command == 'all' then
		print'\127ffffffAll saved loadouts: (- indicates unassigned to station)'
		for name,loadout in pairs(data.presets) do
			local color = SLM.GetLoadoutColor(loadout) or '\127ffffff'
			local is_assigned = loadouts_index[name] and '  ' or '- ' --preface with * if preset not assigned at a station
			print(color..is_assigned..name)
		end
		if not next(data.presets) then print'\127ffffffNo loadouts found'; return end
	elseif command == 'defaults' then
		print'\127ffffffDefault loadout presets:'
		for k,v in pairs(ShipPresets) do
			print('\127ffffff'..k..': '..(InvManager.GetInventoryItemParameter(v.ship, 'Name') or 'NONE'))
		end
	elseif command == 'available' then
		print'\127ffffffLoadouts available for purchase:'
		local match_found = false --assume false until proven otherwise
		for name,loadout in pairs(data.presets) do
			local _, is_avail = SLM.QuickAvailability(loadout)
			if is_avail then
				local color = SLM.GetLoadoutColor(loadout) or '\127ffffff'
				print(color..name)
				match_found = true
			end
		end
		if not match_found then print'\127ffffffNo loadouts found'; return end
	else
		local stationid = GetStationLocation()
		if data.stations[stationid] then
			print'\127ffffffLocal station loadout presets:'
			for i,name in pairs(data.stations[stationid]) do
				local loadout = data.presets[name]
				local color = SLM.GetLoadoutColor(loadout) or ''
				print(i..': '..color..(name or '*'..InvManager.GetInventoryItemParameter(loadout.ship, 'Name'))..'\127o')
			end
		else print'\127ffffffNo loadouts are assigned at this station'; return end
	end
end

--args[1] can be a number (1-4) indicating the slot number to buy or a string indicating the name of the loadout to buy
function SLM:buy(args)
	--TODO Check availability of loadout before purchase when specifying by name
	if args and args[1] then
		local name = args[1] --ignore unless i is nil
		local i = tonumber(args[1]) --nil if not a number
		local stationid = PlayerInStation() and GetStationLocation()
		
		if not stationid or GetStationType() then print'\127ffffffYou must dock at a station first'; return end
		
		if type(i)=='number' then
			if not (i>=1 and i<=4) then print'\127ffffffSlot number must be 1-4'; return end
			if math.floor(i) ~= i then print'\127ffffffSlot number must be an integer'; return end
			
			local name = data.stations[stationid] and data.stations[stationid][i]
			--loadout = data.presets[name]
		--else loadout = data.presets[name] end
		end
		
		local loadout = type(name)=='table' and name or SLM.ui.GetLoadoutByPreset(name)
		
		i = i or 1 --use slot 1 to save loadout if not specified
		if loadout then
			--PurchaseShipLoadout(nil, loadout)
			local default_loadout = ShipPresets[i] --save reference to default loadout
			local is_override = replacePreset(i, loadout) --override default loadout with loadout
			default_actions[i](default_buttons[i]) --make purchase using overriden loadout
			if is_override then replacePreset(i, default_loadout) end --restore default loadout
			
			return true
		else print'\127ffffffSLM: No loadout data found'; return end
	else print'\127ffffffYou must specify a slot (1-4) to buy: e.g. /SSL buy 1'; return end
end

--rename a loadout
function SLM:rename(args)
	if args and args[1] and args[2] then
		local orig_name = args[1]
		local new_name = args[2]
		
		if #args>2 then print'\127ffffffSpecify only 2 parameters. If a name contains spaces then use quotes'; return end
		local is_valid_name, error_msg = SLM.VerifyName(new_name)
		if not is_valid_name then print('\127ffffff'..error_msg); return end
		
		if SLM.data.GetLoadoutByName(orig_name) then
			for stationid,station_presets in pairs(data.stations) do
				for i,name in pairs(station_presets) do
					if name==orig_name then station_presets[i] = new_name end
				end
			end
		
			--swap old with new and delete old data
			data.presets[new_name] = data.presets[orig_name]
			data.presets[orig_name] = nil
			loadouts_index[new_name] = loadouts_index[orig_name]
			loadouts_index[orig_name] = nil
			
			SLM.SaveData()
			
			--GOAL only if necessary
			SLM.replace_station_titles()
	
			print('\127ffffffSLM: '..orig_name..' renamed to '..new_name)
			return true
		else --check if orig_name is special loadout
			local loadout = SLM.ui.GetLoadoutByPreset(orig_name)
			if not loadout then print('\127ffffffLoadout not found: '..orig_name); return end
			
			return SLM.save(self, {"0", new_name, loadout})
		end
	else return SLM.help(self, {'rename'}) end --invalid arguments
end

--verify saved loadout names are valid and prompt user to change if not (version compatibility)
function SLM:update(args)
	local next_name = table.remove(invalid_names) --last item in table if any
	if not next_name then
		--Regenerate list of invalid names
		invalid_names = {}
		for name,loadout in pairs(data.presets) do
			if not SLM.VerifyName(name, true) then table.insert(invalid_names, name) end
		end
		
		next_name = table.remove(invalid_names)
	end
	
	if next_name then
		local rename_cb = function(name)
			if SLM:rename{next_name, name} then SLM:update() end
		end
		
		ShowDialog(SLM.ui.MakeRenameDialog(rename_cb, false, next_name))
	else print'\127ffffffSLM: All saved loadout names are valid'; return end		
end

--set the maximum delay for a double-click (can be specified in seconds or milliseconds)
function SLM:setdoubleclickdelay(args)
	if args and args[1] then
		local delay = tonumber(args[1]) or 0
		
		if delay > 0 then
			if delay <= 2 then
				delay = delay*1000 --convert to milliseconds
			elseif delay > 2000 then print'\127ffffffDouble-click delay cannot be more than 2 seconds'; return end
		else print'\127ffffffDouble-click delay must be positive'; return end
		
		delay = math.floor(delay) --round down
		delay = delay>0 and delay or 1 --must be at least 1 ms
		
		SLM.vars.SaveVars('DoubleClickDelay', delay)
		print('\127ffffffDouble-click delay set to '..(delay/1000)..' seconds')
	else return SLM.help(self, {'doubleclickdelay'}) end
end

--unused
function SLM:debug(args)
end

--console help text
function SLM:help(args)
	args = args or {}
	
	local special_names = '\"ACTIVE SHIP\", \"DEFAULT1\", \"DEFAULT2\", \"DEFAULT3\", \"DEFAULT4\"' --undocumented: \"BASE SHIP: <full_ship_name>\"
	
	local help_text= {
		['help']='\127ffffffStation Loadouts Manager v'..SLM.version..'  Available commands are: (for more info use /SLM help \127ffff00COMMAND\127o)'..
			'\n\t/SLM - Opens the GUI to manage loadouts and set options'..
			'\n\t/SLM save \1270070c0N\127o \127ffff00Name\127o - Saves the current ship\'s loadout to slot \1270070c0N\127o (1-4) at the current station with the specified preset name'..
			'\n\t/SLM assign \1270070c0N\127o \127ffff00Name\127o - Saves previously saved loadout with \127ffff00Name\127o to slot \1270070c0N\127o at this station'..
			'\n\t/SLM rename \127ffff00Old_Name\127o \127ffff00New_Name\127o - Renames loadout \127ffff00Old_Name\127o to \127ffff00New_Name\127o'..
			'\n\t/SLM remove \1270070c0N\127o - Removes loadout in slot \1270070c0N\127o from the current station (universe-wide default will be used)'..
			'\n\t/SLM delete \127ffff00Name\127o - Deletes loadout with \127ffff00Name\127o and removes it from all stations'..
			'\n\t/SLM list \127808080[\127ff00ffall\127808080|\12700ff00available\127808080|\12700ffffdefaults\127808080]\127ffffff - Display loadouts for the current station'..
			'\n\t/SLM buy \127ffff00Name\127o - Purchase loadout \127ffff00Name\127o'..
			'\n\t/SLM update - Verifies loadout names with current version of SLM. You will be prompted to rename any incompatable names found',
		['save']='\127ffffff/SLM save \1270070c0N\127o \127ffff00Name\127o - Saves the current ship\'s loadout to slot \1270070c0N\127o (1-4) at the current station with the specified preset name'..
			'\n\tUse \1270070c0N\127o=0 to create a new loadout with specified name but not assign it to the station'..
			'\n\tUse \1270070c0N\127o=DEFAULT\1270070c0N\127o (1-4) to save the loadout to a default preset instead of a station preset (\127ffff00Name\127o is ignored)',
		['assign']='\127ffffff/SLM assign \1270070c0N\127o \127ffff00Name\127o - Saves previously saved loadout with \127ffff00Name\127o to slot \1270070c0N\127o at this station'..
			'\n\tUse \1270070c0N\127o=DEFAULT\1270070c0N\127o (1-4) to save the loadout to a default preset instead of a station preset',
		['rename']='\127ffffff/SLM rename \127ffff00Old_Name\127o \127ffff00New_Name\127o - Renames loadout \127ffff00Old_Name\127o to \127ffff00New_Name\127o'..
			'\n\tThis will rename all station presets using the loadout as well'..
			'\n\tIn addition to saved loadout names, \127ffff00Old_Name\127o can also use the following special names: '..special_names,
		['remove']='\127ffffff/SLM remove \1270070c0N\127o - Removes loadout in slot \1270070c0N\127o from the current station (universe-wide default will be used instead)'..
			'\n\tThe removed loadout can still be used later. To completely delete it see also: [/slm help delete]',
		['delete']='\127ffffff/SLM delete \127ffff00Name\127o - Deletes loadout with \127ffff00Name\127o and removes it from all stations'..
			'\n\tSee also: [/slm help remove] to remove a loadout as a station preset but still keep the loadout for later use',
		['list']='\127ffffff/SLM list \127808080[\127ff00ffall\127808080|\12700ff00available\127808080|\12700ffffdefaults\127808080]\127ffffff - Display loadouts for the current station'..
			'\n\tOptional parameter \127ff00ffall\127o lists the names of every saved loadout'..
			'\n\tOptional parameter \12700ff00available\127o lists loadouts currently available to buy at the current station'..
			'\n\tOptional parameter \12700ffffdefaults\127o lists the 4 universe-wide defaults (used if the slot isn\'t assigned a station preset)',
		['buy']='\127ffffff/SLM buy \127ffff00Name\127o - Purchase loadout \127ffff00Name\127o'..
			'\n\tIn addition to saved loadout names, \127ffff00Name\127o can also use the following special names: '..special_names,
		['update']='\127ffffff/SLM update - Verifies loadout names with current version of SLM. You will be prompted to rename any incompatable names found',
		['setdoubleclickdelay']='\127ffffff/SLM setdoubleclickdelay \1270070c0N\127o - Set the maximum delay, \1270070c0N\127o, for a double-click in seconds or milliseconds (2 seconds max)'..
			'\n\tDelay is currently set to '..((SLM.vars.GetVars"DoubleClickDelay" or 0)/1000)..' seconds',
		['command']='\127ffffffTo get more information about how to use a command, type: /SLM help \127ffff00COMMAND\127o'..
			'\n\tWhere \127ffff00COMMAND\127o is the command you want to get help on (e.g. /SLM help save)',
	}
	
	print(args[1] and help_text[args[1]:lower()] or help_text.help)
end

RegisterUserCommand('slm', SLM.input)
RegisterEvent(SLM.event, 'ENTERED_STATION') --to update buyback dialog
RegisterEvent(SLM.event, 'PLAYER_ENTERED_GAME') --to update buyback dialog