local ui = {}
ui.COLOR = {
	WHITE='\127ffffff',
	GREY='\127a0a0a0',
	YELLOW='\127ffff00',
	RED='\127ff0000',
}
ui.MAX_NUM_PORTS = 7
ui.LastDoubleClick = {}

--* Begin UI *--

--Add button to PDA to launch GUI
local main_button = iup.stationbutton{title="Open SLM", action=function() SLM.ui:Show() end} --PDA button to launch GUI
local loc = StationEquipmentTab[1][1][1][1][1][2][1][2][1][1][1][1][1][1][3][1][1][3]

loc[2].size=2
iup.Append(loc, main_button)
iup.Append(loc, iup.fill{})

--Override Purchase Ship Button
local old_purchase_action = StationEquipmentBuyShipPurchaseButton.action
StationEquipmentBuyShipPurchaseButton.action = function(self, ...)
	local is_not_tutorial = not ShouldTutorialRun() --don't show enhanced purchase dialog during tutorial
	if PlayerInStation() and is_not_tutorial and SLM.vars.GetVars"IsEnhancedPurchase" then ui.purchase:Show() else old_purchase_action(self, ...) end
end

-- Common Functions --

local filter_compare = {
	['name'] = function(_, name)
		local name_str = type(name)=='string' and name:gsub("%s+",'') or ''
		return {SLM.vars.GetVars"IsCaseInsensitive" and name_str:lower() or name_str}
	end,
	['ship'] = function(loadout, _)
		local ship = (InvManager.GetInventoryItemParameter(loadout.ship, 'Name') or ''):gsub("%s+",'')
		return {SLM.vars.GetVars"IsCaseInsensitive" and ship:lower() or ship}
	end,
	['addons'] = function(loadout, _)
		local tbl = {}
		for _,itemid in pairs(loadout.ports) do
			local addon = (InvManager.GetInventoryItemParameter(itemid, 'Name') or ''):gsub("%s+",'')
			if addon then table.insert(tbl, SLM.vars.GetVars"IsCaseInsensitive" and addon:lower() or addon) end
		end
		return tbl
	end,
	['color']=function(loadout, _) return {tostring(loadout.shipcolor)} end,
	--['rgb']=function(loadout, _) return {ShipPalette_string[loadout.shipcolor+1]} end, --TODO; index shift: ShipPallette is 1-256, not 0-255
	['sector']=function(_, name)
		local tbl = {}
		for stationid,_ in pairs(SLM.data.GetStationsByLoadout(name) or {}) do
			local sector = ShortLocationStr(GetSectorIDOfStation(stationid)):gsub("[%s%-]",'') --get sector and remove whitespace and - between col/row
			table.insert(tbl, SLM.vars.GetVars"IsCaseInsensitive" and sector:lower() or sector)
		end
		return tbl
	end,
}

local sort_funcs = {
	['Loadout Name']=function(a,b) return a.name_lower < b.name_lower end,
	['Ship Name']=function(a,b) --sort by ship name, loadout name
		if a.ship_name == b.ship_name then
			return a.name_lower < b.name_lower
		else return a.ship_name < b.ship_name end
	end,
	['Availability']=function(a,b) --sort by availability, loadout name		
		if not a.price == not b.price then --a has same availability as b
			if not a.avail == not b.avail then
				return a.name_lower < b.name_lower --sort then by loadout name
			else return not b.avail end
		else return not b.price end --whichever one is available comes first
	end,
	['Price']=function(a,b) --sort by availability, price, loadout name
		if a.price and b.price then --both a and b available
			if not a.avail == not b.avail then
				return a.price < b.price --sort by price
			else return not b.avail end
		elseif a.price or b.price then --one available, one isn't
			return not b.price --whichever one is available comes first
		else return a.name_lower < b.name_lower end --neither available, sort by loadout name
	end,
	['Color']=function(a,b) --sort by color, loadout_name
		local color_a = a.loadout.shipcolor or 0
		local color_b = b.loadout.shipcolor or 0
		
		if color_a ~= color_b then
			return color_a < color_b
		else return a.name_lower < b.name_lower end
	end,
}

function ui:doRename()
	local preset_name = ui.FilterPresetName(ui.GetListSelected(self.elements.list))
	
	local rename_cb
	if preset_name~=nil then
		ui.LastDoubleClick.timestamp = nil --prevent false double-click
		if SLM.data.GetLoadoutByName(preset_name) then
			--rename saved loadout
			rename_cb = function(name)
				if SLM.rename(self, {preset_name, name}) then
					--Regenerate Preset button text --GOAL only update text that changed
					if PlayerInStation() and not GetStationType() then --not in capship
						local stationid = GetStationLocation()
						local station_presets = SLM.data.GetLoadoutByStation(stationid) or {}
			
						for i=1,4 do
							local name_i = station_presets[i]
							if name_i then
								local color = SLM.GetLoadoutColor(SLM.data.GetLoadoutByName(name_i)) or ''
								self.elements.preset_labels[i].title = color..name_i
							else self.elements.preset_labels[i].title = "UNASSIGNED" end
						end
					end
					
					--update name in list
					local list_sel = self.elements.list.vo_listsel
					local color = SLM.GetLoadoutColor(SLM.data.GetLoadoutByName(name)) or ui.COLOR.WHITE
					self.elements.list[list_sel] = color..name --change name in list
					ui.UpdateListSelection(self.elements.list) --update loadout info panel
					iup.Refresh(self.elements.dialog or iup.GetDialog(self))
				end
			end
		else
			--rename special loadout (e.g. active ship)
			rename_cb = function(name)
				local loadout = ui.GetLoadoutByPreset(preset_name)
				if not loadout then return end --should be redundant, disable button if not valid loadout
				
				if SLM.save(self, {0, name, loadout}) then
					--GOAL insert new entry in list and select it
					ui.LoadPresets(self)
					self.elements.list.value = ui.GetListItemIndex(self.elements.list, name) or 1
					ui.UpdateListSelection(self.elements.list)
				end
			end
		end
	end
		
	local rename_dlg = ui.MakeRenameDialog(rename_cb, false, preset_name)
	ShowDialog(rename_dlg)
end

function ui:doDelete()
	local function ok_cb()
		local preset_name = ui.FilterPresetName(ui.GetListSelected(self.elements.list))
		
		if preset_name~=nil then
			if SLM.delete(self, {preset_name}) then
				ui.LoadPresets(self) --GOAL More elegant method
			end
		end
		
		HideDialog(QuestionDialog)
	end

	local function cancel_cb()
		HideDialog(QuestionDialog)
	end

	QuestionDialog:SetMessage("Are you sure you want to delete this loadout?\nIt will be removed from ALL stations", "Ok", ok_cb, "Cancel", cancel_cb)
	ShowDialog(QuestionDialog)
end

function ui:doBuy()
	local preset_name = ui.FilterPresetName(ui.GetListSelected(self.elements.list))
	local base_ship_name = iup.GetType(self.elements.filter_search)=='list' and ui.FilterPresetName(self.elements.filter_search[self.elements.filter_search.value]) or ''
	local loadout = ui.GetLoadoutByPreset(preset_name=="BASE SHIP" and 'BASE SHIP: '..base_ship_name or preset_name) --colon prevents conflict with existing loadout name
	
	if not PlayerInStation() then return false end --must be in station to make purchase TODO error message, should be redundant
	
	local function makePurchase() --need separate function from ok_cb to prevent conflict with question dialog for not enough storage space
		HideDialog(iup.GetDialog(self))
		SLM.buy(self, {loadout})
	end
	
	local function ok_cb()
		HideDialog(QuestionDialog) --must call before makePurchase()
		makePurchase()
	end
	local function cancel_cb() HideDialog(QuestionDialog) end
	
	local _,is_avail = SLM.QuickAvailability(loadout)
	if not is_avail then
		QuestionDialog:SetMessage("Not all items for this loadout are available at this station.\nAre you sure you want to buy it?", "Ok", ok_cb, "Cancel", cancel_cb)
		ShowDialog(QuestionDialog)
	else makePurchase() end --make purchase w/o confirmation
end

--Assign presets/defaults button actions
ui.doPresets = {}
ui.doDefaults = {}
for i=1,4 do
	--ui.presets.element.preset_buttons[i].action = function()
	ui.doPresets[i] = function(self)
		local preset_name = ui.FilterPresetName(ui.GetListSelected(ui.presets.element.preset_list))
		local loadout = ui.GetLoadoutByPreset(preset_name)
		local color = SLM.GetLoadoutColor(loadout) or ''
		
		if SLM.data.GetLoadoutByName(preset_name)~=nil then
			if SLM.assign(self, {i, preset_name}) then
				self.labels.title = color..(preset_name or "#ERROR")
			end
		else
			local assign_preset_cb = function(name)
				if SLM.save(self, {i, name, loadout}) then
					self.labels.title = color..(name or "#ERROR")
					
					local station_preset = SLM.data.GetLoadoutByStation(nil, i)
					if station_preset then
						local color = SLM.GetLoadoutColor(SLM.data.GetLoadoutByName(station_preset)) or ''
						self.labels.title = color..station_preset
					else self.labels.title = "UNASSIGNED" end
					
					ui.LoadPresets(self) --to add to list --OBSOLETE
				end
			end
			
			local rename_dlg = ui.MakeRenameDialog(assign_preset_cb, false)
			ShowDialog(rename_dlg)
		end
		
		iup.Refresh(self.elements.dialog or iup.GetDialog(self))
	end
	
	--ui.presets.element.default_buttons[i].action = function()
	ui.doDefaults[i] = function(self)
		local preset_name = ui.FilterPresetName(ui.GetListSelected(ui.presets.element.preset_list))
		local loadout = ui.GetLoadoutByPreset(preset_name)
		
		if SLM.assign(self, {"DEFAULT"..i, preset_name}) then
			ui.presets.element.default_labels[i].title = ui.COLOR.GREY..(SLM.GetTruncatedShipName(loadout) or "#ERROR")
		end
		
		iup.Refresh(self.elements.dialog or iup.GetDialog(self))
	end
end

function ui:doOnlyAvailableToggle()
	ui.varsToggle.action(self)
	ui.LoadPresets(self)
end

--For selecting search from dropdown list (e.g. purchase dialog)
function ui:doListSearch(item, n, sel)
	if sel==1 then ui.LoadPresets(self) end
end

--Action when list item is selected
function ui:doList(item, n, sel)
	if sel==1 then ui.UpdateListSelection(self, item, n) end
end

--Update displated GUI loadout info  based on the selected list item
function ui:UpdateListSelection(item, n)
	item = item or ui.GetListSelected(self)
	n = n or self.value
	
	self.elements.loadout_name.title = item
	local preset_name = ui.FilterPresetName(item or '')
	local base_ship_name = iup.GetType(self.elements.filter_search)=='list' and ui.FilterPresetName(self.elements.filter_search[self.elements.filter_search.value]) or ''
	local loadout = ui.GetLoadoutByPreset(preset_name=="BASE SHIP" and 'BASE SHIP: '..base_ship_name or preset_name) --colon prevents conflict with existing loadout name
	
	if loadout then
		local price, is_avail, addon_avail = SLM.CheckAvailability(loadout)
		
		self.elements.ship.img.image = InvManager.GetInventoryItemParameter(loadout.ship, 'Icon') or ''
		self.elements.ship.img.visible = self.elements.ship.img.image=='' and "NO" or "YES"
		local ship_avail_color = (not PlayerInStation() or (addon_avail and addon_avail[loadout.ship]=='BUY')) and ui.COLOR.WHITE or ui.COLOR.RED
		self.elements.ship.name.title = ship_avail_color..(InvManager.GetInventoryItemParameter(loadout.ship, 'Name') or '')
		
		local is_over_budget = price and GetMoney() < price
		local price_color = (is_over_budget or not price) and ui.COLOR.RED or (not is_avail and ui.COLOR.YELLOW or (is_avail=='STORAGE' and ui.COLOR.GREY or ui.COLOR.WHITE))
		self.elements.price.title = string.gsub(comma_value(price) or "NOT AVAILABLE", '([%d,]+)$', price_color..'%1 cr')
		self.elements.buy_button.active = PlayerInStation() and not GetStationType() and not is_over_budget and "YES" or "NO"
		
		--purchase loadout if double-clicked and feature enabled
		if SLM.vars.GetVars"IsDoubleClickBuy" and self.elements.buy_button.active=="YES" then
			if ui.LastDoubleClick.loadout_name==item and ui.LastDoubleClick.timestamp and ui.LastDoubleClick.timestamp>0 then
				local duration = gkmisc.GetGameTime()-ui.LastDoubleClick.timestamp
				if duration < SLM.vars.GetVars"DoubleClickDelay" and duration > 0 and self.elements.buy_button.action then
					self.elements.buy_button.action(self.elements.buy_button)
				else ui.LastDoubleClick.timestamp=gkmisc.GetGameTime() end
			else ui.LastDoubleClick = {loadout_name=item, timestamp=gkmisc.GetGameTime()} end --save timestamp for last loadout clicked
		end
		
		--GOAL make more elegant
		--assign addon names - start with blank slate
		for i=1,ui.MAX_NUM_PORTS do
			self.elements.addons.imgs[i].image = ''
			self.elements.addons.imgs[i].visible = "NO"
			self.elements.addons.names[i].title = ui.COLOR.WHITE..'none'
		end
		for i,itemtypeid in ipairs(loadout.ports) do
			local addon_name = InvManager.GetInventoryItemParameter(itemtypeid, 'Name') or 'none'
			self.elements.addons.imgs[i].image = InvManager.GetInventoryItemParameter(itemtypeid, 'Icon') or ''
			self.elements.addons.imgs[i].visible = self.elements.addons.imgs[i].image=='' and "NO" or "YES"
			local addon_color = (not PlayerInStation() or (addon_avail and addon_avail[itemtypeid]=='BUY' or itemtypeid==0)) and ui.COLOR.WHITE or (addon_avail and addon_avail[itemtypeid]=='STORAGE' and ui.COLOR.GREY or ui.COLOR.RED)
			self.elements.addons.names[i].title = addon_color..addon_name
		end
		
		for i=1,4 do
			self.elements.assign_presets[i].active = PlayerInStation() and "YES" or "NO"
			self.elements.assign_defaults[i].active = "YES"
		end
		
		self.elements.rename_button.active = "YES"
	else --e.g. no active ship
		self.elements.ship.img.image = ''
		self.elements.ship.img.visible = "NO"
		self.elements.ship.name.title = "No Ship"
		self.elements.price.title = "NOT AVAILABLE"
		for i=1,ui.MAX_NUM_PORTS do
			self.elements.addons.imgs[i].image = ''
			self.elements.addons.imgs[i].visible = "NO"
			self.elements.addons.names[i].title = 'none'
		end
		
		--disable buttons because loadout is invalid
		for i=1,4 do
			self.elements.assign_presets[i].active = "NO"
			self.elements.assign_defaults[i].active = "NO"
		end
		self.elements.rename_button.active = "NO"
		self.elements.buy_button.active = "NO"
	end
	
	--disable delete buttons for special loadouts
	if SLM.data.GetLoadoutByName(preset_name) then
		self.elements.delete_button.active = "YES"
	else self.elements.delete_button.active = "NO" end
	
	--iup.Map(self.elements.dialog or iup.GetDialog(self)) --neccessary?
	iup.Refresh(self.elements.dialog or iup.GetDialog(self))
end

-- Main GUI --

ui.element = {} --iup controls
ui.pane = {} --groupings of iup controls

ui.element.close_button = iup.stationbutton{title="Close", expand="HORIZONTAL"}

function ui.BEGIN_LOADOUTS_TAB() end

-- Presets Tab --

ui.presets = {element={}, pane={}}

ui.presets.element.name_label = iup.label{title="Loadout Name: ", expand="No"}
ui.presets.element.loadout_name = iup.label{title="", expand="HORIZONTAL"}
ui.presets.element.ship_label = iup.label{title="Ship: ", expand="No"}
ui.presets.element.ship_img = iup.label{title="", size="32x32", image="", visible="NO"}
ui.presets.element.ship = iup.label{title="REALLYREALLYLONGNAME", expand="HORIZONTAL"}

ui.presets.element.addons_label = iup.label{title="Addons:", expand="No"}
ui.presets.element.addons_box = {}
ui.presets.element.addons = {}
ui.presets.element.addon_imgs = {}
for i=1,ui.MAX_NUM_PORTS do
	ui.presets.element.addon_imgs[i] = iup.label{title="", size="32x32", image="", visible="NO"}
	ui.presets.element.addons[i] = iup.label{title="none", expand="HORIZONTAL"}
	
	ui.presets.element.addons_box[i] = iup.hbox{
		ui.presets.element.addon_imgs[i],
		iup.vbox{ iup.fill{}, ui.presets.element.addons[i], iup.fill{} },
	}
end

ui.presets.element.price_label = iup.label{title="Price: "}
ui.presets.element.price = iup.label{title=""}
ui.presets.element.sector = iup.label{title=""}
ui.presets.element.station_name = iup.label{title=""}

ui.presets.element.rename_button = iup.stationbutton{title="Rename", action=ui.doRename}
ui.presets.element.delete_button = iup.stationbutton{title="Delete Loadout", action=ui.doDelete}
--ui.presets.element.save_color_button = iup.stationbutton{title="Save Color"} --TODO
ui.presets.element.buy_button = iup.stationbutton{title="Buy Loadout", action=ui.doBuy}

ui.presets.element.default_buttons = {}
ui.presets.element.default_labels = {}
ui.presets.element.preset_buttons = {}
ui.presets.element.preset_labels = {}
ui.pane.preset_buttons = iup.hbox{gap=2}
for i=1,4 do
	local width = math.floor(189+((gkinterface.GetXResolution() or 800) - 800)/4)
	ui.presets.element.default_buttons[i] = iup.stationbutton{title="Assign Default "..i, size=width.."x", action=ui.doDefaults[i]}
	ui.presets.element.default_labels[i] = iup.label{title="", size=width.."x", alignment="ACENTER"}
	ui.presets.element.preset_buttons[i] = iup.stationbutton{title="Assign Preset "..i, size=width.."x", action=ui.doPresets[i]}
	ui.presets.element.preset_labels[i] = iup.label{title="", size=width.."x", alignment="ACENTER"}
	
	ui.presets.element.default_buttons[i].labels = ui.presets.element.default_labels[i] --non-standard attribute
	ui.presets.element.preset_buttons[i].labels = ui.presets.element.preset_labels[i] --non-standard attribute
	
	iup.Append(ui.pane.preset_buttons, iup.vbox{ui.presets.element.preset_buttons[i], ui.presets.element.preset_labels[i], ui.presets.element.default_buttons[i], ui.presets.element.default_labels[i]})
end

ui.presets.element.preset_list = iup.pdasubsubsublist{value=0, minsize="240x360", size="240x", expand="YES", action=ui.doList} --240x360
ui.presets.element.showOnlyAvailable = iup.stationtoggle{title="Show only available loadouts", key="IsOnlyAvailable"}
ui.presets.element.filter_search = iup.text{value="", expand="HORIZONTAL"}
ui.presets.element.filter_button = iup.stationbutton{title="Go"}
ui.presets.element.filter_type = iup.pdasubsubsublist{
	"Name", "Ship", "Addons", "Sector", "Advanced",
	value=1,
	dropdown="YES",
	--size="150x",
	expand="NO",
}

ui.presets.pane.preset_info = iup.vbox{
	iup.hbox{ui.presets.element.name_label, ui.presets.element.loadout_name},
	iup.hbox{
		ui.presets.element.ship_label,
		ui.presets.element.ship_img,
		iup.vbox{ iup.fill{}, ui.presets.element.ship, iup.fill{} },
	},
	iup.fill{size=18},
	ui.presets.element.addons_label,
	iup.hbox{iup.vbox(ui.presets.element.addons_box),iup.fill{}}, --TODO add color picker
	iup.fill{size=18},
	iup.hbox{ui.presets.element.price_label, ui.presets.element.price},
	iup.fill{},
	expand="NO",
}

ui.presets.tab = iup.vbox{
	iup.hbox{
		ui.presets.pane.preset_info,
		iup.fill{},
		iup.vbox{iup.hbox{iup.label{title="Filter by: "}, ui.presets.element.filter_type}, ui.presets.element.showOnlyAvailable, iup.fill{}},
		iup.hbox{ iup.vbox{
			iup.hbox{ui.presets.element.filter_search, ui.presets.element.filter_button},
			ui.presets.element.preset_list,
			gap=2,
		}, size="240x", expand="VERTICAL"},--direction="WEST"},
		gap=8,
	},
	iup.hbox{ui.presets.element.sector, ui.presets.element.station_name, iup.fill{}, ui.presets.element.rename_button, ui.presets.element.delete_button, iup.fill{size=12}, ui.presets.element.buy_button},
	iup.fill{size=6},
	iup.hbox{ui.pane.preset_buttons, iup.fill{size=4}},
	--margin="10x20",
	tabtitle="Loadouts",
	hotkey=iup.K_L,
}

--create reference to controls for associated loadout display
ui.presets.display = {
	list = ui.presets.element.preset_list,
	station_list = {},
	only_available = ui.presets.element.showOnlyAvailable,
	filter_search = ui.presets.element.filter_search,
	filter_type = ui.presets.element.filter_type,
	
	loadout_name = ui.presets.element.loadout_name,
	ship = {name=ui.presets.element.ship, img=ui.presets.element.ship_img},
	addons = {names=ui.presets.element.addons, imgs=ui.presets.element.addon_imgs},
	price = ui.presets.element.price,
	
	assign_presets = ui.presets.element.preset_buttons,
	preset_labels = ui.presets.element.preset_labels,
	assign_defaults = ui.presets.element.default_buttons,
	default_labels = ui.presets.element.default_labels,
	
	sector_name = ui.presets.element.sector,
	station_name = ui.presets.element.station_name,
	
	buy_button=ui.presets.element.buy_button,
	rename_button = ui.presets.element.rename_button,
	delete_button = ui.presets.element.delete_button,
	
	default_focus = ui.presets.element.preset_list,
	
	dialog = ui.presets.tab,
}
ui.presets.element.preset_list.elements = ui.presets.display --non-standard attribute
ui.presets.element.rename_button.elements = ui.presets.display --non-standard attribute
ui.presets.element.delete_button.elements = ui.presets.display --non-standard attribute
ui.presets.element.buy_button.elements = ui.presets.display --non-standard attribute
ui.presets.element.showOnlyAvailable.elements = ui.presets.display --non-standard attribute
ui.presets.element.filter_button.elements = ui.presets.display --non-standard attribute
for i=1,4 do
	ui.presets.element.preset_buttons[i].elements = ui.presets.display --non-standard attribute
	ui.presets.element.default_buttons[i].elements = ui.presets.display --non-standard attribute
end
ui.presets.tab.elements = ui.presets.display --non-standard attribute

function ui.presets.tab:OnHelp()
	StationHelpDialog:Open(
[[You can manage your list of saved ship loadouts from this dialog window.

The list on the right shows all your saved loadouts. Select a loadout to see info about which ship and addons it uses (shown on the left side of the dialog). Loadouts in which not all items are available at the current station are prefaced with *

The buttons apply to the loadout selected from the list:
*Rename: Allows you to change the name of an existing loadout at all stations where it is used (20 characters max)
*Delete Loadout: Removes the selected loadout and deletes it from all stations where it is currently assigned (default loadouts will be used instead). Special loadouts (DEFAULT loadouts and the ACTIVE SHIP loadout) cannot be deleted.
*Buy Loadout: Purchases the selected loadout and makes it your active ship. The ship and addons must be available at the current station (either to buy or in storage).

You can use the buttons along the bottom to assign a loadout as a preset for the current station. The 'Assign Preset #' button will assign the loadout to the current station in slot #. You can also assign it as a default loadout to all stations with the 'Assign Default #' button. Stations that don't have a preset assigned will use the defaults instead.

The filter bar above the loadouts list allows you to search for loadouts matching certain criteria. Select the search type from the dropdown menu. Entries that contain the search text will be returned. Options are:
NAME - Search by the name of the saved loadout.
SHIP - Search by the name of the ship used in the loadout (e.g. ec-88). WARNING: the - character has special meaning if pattern matching is enabled. Instead use %- or . to represent any character.
ADDONS - Search by an addon used in the loadout (e.g. corvuswidowmaker). Separate multiple addons by commas and the entries returned must be equiped with all the addons listed.
SECTOR - Search by a sector where the loadout is saved as a station preset (e.g. artacaelestisj11). Separate multiple sectors by commas and the entries returned must match all sectors listed.
ADVANCED - Search multiple types. Example: "ship:vult addons:neut,posi,ultracharge sector:sedina"

Colors of addons text:
WHITE - Addon is available for purchase
GREY - Addon is not available for purchase but can be found in storage
RED - Addon is not available for purchase nor in storage

Colors of price text:
WHITE - Ship and all addons are available for purchase
GREY - At least one addon is not available for purchase but can be found in storage
YELLOW - Not all of the addons are available, and a partial loadout will be purchased
RED - The ship is not available or you do not have sufficient credits for the purchase

Loadout Naming Conventions:
*Name cannot be longer than 20 characters
*Name must contain at least one non-numeric character
*Name cannot begin with * or #
*Name cannot contain , or :
*Each loadout must have a unique name (but a loadout can be assigned as a preset at multiple stations)
*There are additional reserved names that cannot be used (reserved names are in all caps)

Loadout data is saved in ]].."/settings/"..(GetPlayerName() or 'NAME').."/system"..SLM.NOTES.main.."notes.txt")
end

function ui.BEGIN_OPTIONS_TAB() end

-- Options Tab --

ui.options = {element={}, pane={}}

ui.options.sort_types = {"Loadout Name", "Ship Name", "Availability", "Price", "Color",}
ui.varsToggle = {}
function ui.varsToggle:action()
	local bool = self.value=="ON"
	SLM.vars.SaveVars(self.key, bool)
	
	if self.key=="IsUseColor" then
		SLM.replace_station_titles()
		--SLM.replace_buyback_titles()
	end
end

ui.options.element.use_color_toggle = iup.stationtoggle{title="Use ship color for loadout names", key="IsUseColor", action=ui.varsToggle.action}
ui.options.element.enhanced_purchase_toggle = iup.stationtoggle{title="Use enhanced purchase ship dialog", key="IsEnhancedPurchase", action=ui.varsToggle.action}
ui.options.element.double_click_buy_toggle = iup.stationtoggle{title="Double-click to purchase loadout", key="IsDoubleClickBuy", action=ui.varsToggle.action}
ui.options.element.case_insensitive_toggle = iup.stationtoggle{title="Case insensitive searches", key="IsCaseInsensitive", action=ui.varsToggle.action}
ui.options.element.pattern_matching_toggle = iup.stationtoggle{title="Use pattern matching in searches", key="IsPatternMatching", action=ui.varsToggle.action}
ui.options.element.gmt_toggle = iup.stationtoggle{title="Display times in GMT", key="IsGMT", action=ui.varsToggle.action}
ui.options.element.backup_button = iup.stationbutton{title="Backup Loadouts"}
ui.options.element.restore_button = iup.stationbutton{title="Restore Loadouts"}
ui.options.element.backup_info = iup.label{title="", FGCOLOR="150 150 150"}
ui.options.element.restore_info = iup.label{title="", FGCOLOR="150 150 150"}
ui.options.element.backup_file_loc = iup.label{title="/settings/"..(GetPlayerName() or 'NAME').."/system"..SLM.NOTES.bak.."notes.txt"}
ui.options.pane.backup = iup.vbox{
	iup.label{title="Backup & Restoration"},
	iup.hbox{ui.options.element.backup_button, ui.options.element.backup_info, iup.fill{}, gap=4},
	iup.hbox{ui.options.element.restore_button, ui.options.element.restore_info, iup.fill{}, gap=4},
	ui.options.element.backup_file_loc,
}

--sort presets popup
ui.options.element.sort_presets_popup = iup.list{dropdown="YES"}

for i,v in ipairs(ui.options.sort_types) do ui.options.element.sort_presets_popup[i]=v end

ui.options.tab = iup.vbox{
	ui.options.element.use_color_toggle,
	iup.hbox{iup.label{title="Sort Loadouts By: "}, ui.options.element.sort_presets_popup, iup.fill{}},
	iup.fill{size=30},
	ui.options.element.enhanced_purchase_toggle,
	iup.fill{size=30},
	ui.options.element.double_click_buy_toggle,
	iup.fill{size=30},
	ui.options.element.case_insensitive_toggle,
	ui.options.element.pattern_matching_toggle,
	iup.fill{size=30},
	ui.options.pane.backup,
	iup.fill{size=30},
	ui.options.element.gmt_toggle,
	tabtitle="Options",
	hotkey=iup.K_O,
}

function ui.options.element.sort_presets_popup:action(item, n, sel)
	if sel==1 then SLM.vars.SaveVars("SortPresetsBy", n) end
end

function ui.options.element.backup_button:action()
	if SLM.SaveData(SLM.NOTES.bak) then
		ui.options.element.backup_info.title = "BACKUP SAVED SUCCESSFULLY"
	else ui.options.element.backup_info.title = "ERROR" end
	
	iup.Refresh(ui.options.tab)
end

function ui.options.element.restore_button:action()
	local function ok_cb()
		if SLM.LoadData(SLM.NOTES.bak) then
			SLM.SaveData() --override existing notes
			ui.options.element.restore_info.title = "LOADOUTS RESTORED SUCCESSFULLY"
		else
			SLM.LoadData() --reload original saved data
			ui.options.element.restore_info.title = "NO BACKUP DATA FOUND"
		end
		
		iup.Refresh(ui.options.tab)
		HideDialog(QuestionDialog)
	end

	local function cancel_cb()
		HideDialog(QuestionDialog)
	end
	
	local bak_data = LoadSystemNotes(SLM.NOTES.bak)
	if bak_data~='' then --verify backup data exists
		local timestamp = unspickle(bak_data).timestamp
		local date = timestamp and os.date((SLM.vars.GetVars"IsGMT" and '!' or '').."%c", timestamp) or "UNKNOWN"
		QuestionDialog:SetMessage("Do you want to restore loadout data saved on "..date.."\nYour existing loadout data will be lost", "Ok", ok_cb, "Cancel", cancel_cb)
		ShowDialog(QuestionDialog)
	else
		ui.options.element.restore_info.title = "NO BACKUP DATA FOUND"
		iup.Refresh(ui.options.tab)
	end
end

function ui.options.tab:OnHelp()
	StationHelpDialog:Open(
[[Various settings for ShipLoadoutsManager can be configured from this dialog. These settings are saved to the config file to persist after logout.

Options:
*Use ship color for loadout names: if this option is checked then the text color for the loadout name will match the color of the ship used in the loadout
*Sort Loadouts By: Select a sorting option from this dropdown menu to arrange the order that loadouts are listed.
*Use enhanced purchase ship dialog: Allows you to select a loadout for a given ship when you buy it.
*Case insensitive searches: case ignored in a search (recommended enabled). You will need to disable it for pattern matching that uses capital characters
*Use pattern matching in searches: Allows for lua pattern matching. If you don't know what this is then disable it.
*Backup Loadouts: Saves a second copy of all your loadout data that can restored in case you accidentaly delete a loadout or make some other error.
*Restore Loadouts: Reverts loadout data back to the last saved backup. All data from after the backup was made will be lost.
]])
end

function ui.BEGIN_EXPORT_TAB() end

-- IMPORT/EXPORT Tab --

--[====[ --TODO
ui.export = {element={}, pane={}}

ui.export.element.name_label = iup.label{title="Loadout Name: ", expand="No"}
ui.export.element.loadout_name = iup.label{title="", expand="HORIZONTAL"}
ui.export.element.ship_label = iup.label{title="Ship: ", expand="No"}
ui.export.element.ship_img = iup.label{title="", size="32x32", image="", visible="NO"}
ui.export.element.ship = iup.label{title="REALLYREALLYLONGNAME", expand="HORIZONTAL"}

ui.export.element.addons_label = iup.label{title="Addons:", expand="No"}
ui.export.element.addons_box = {}
ui.export.element.addons = {}
ui.export.element.addon_imgs = {}
for i=1,ui.MAX_NUM_PORTS do
	ui.export.element.addon_imgs[i] = iup.label{title="", size="32x32", image="", visible="NO"}
	ui.export.element.addons[i] = iup.label{title="none", expand="HORIZONTAL"}
	
	ui.export.element.addons_box[i] = iup.hbox{
		ui.export.element.addon_imgs[i],
		iup.vbox{ iup.fill{}, ui.export.element.addons[i], iup.fill{} },
	}
end

ui.export.element.preset_list = iup.pdasubsubsublist{value=0, size="240x", expand="YES", action=ui.doList}
ui.export.element.export_list = iup.pdasubsubsublist{value=0, size="240x", expand="YES", action=ui.doList} --TODO needs own action?

ui.export.element.import_button = iup.stationbutton{title="<"}
ui.export.element.import_all = iup.stationbutton{title="<<"}
ui.export.element.export_button = iup.stationbutton{title=">"}
ui.export.element.export_all = iup.stationbutton{title=">>"}

ui.export.pane.preset_info = iup.vbox{
	iup.hbox{ui.export.element.name_label, ui.export.element.loadout_name},
	iup.hbox{
		ui.export.element.ship_label,
		ui.export.element.ship_img,
		iup.vbox{ iup.fill{}, ui.export.element.ship, iup.fill{} },
	},
	iup.fill{size=18},
	ui.export.element.addons_label,
	iup.hbox{iup.vbox(ui.export.element.addons_box),iup.fill{}},
	iup.fill{size=18},
	iup.hbox{ui.export.element.price_label, ui.export.element.price},
	iup.fill{},
	expand="NO",
}

ui.export.tab = iup.vbox{
	iup.hbox{
		ui.export.pane.preset_info,
		iup.hbox{ iup.vbox{
			ui.presets.element.preset_list,
			gap=2,
		}, size="240x", expand="VERTICAL"},--direction="WEST"},
		gap=8,
	},
	iup.fill{size=6},
	tabtitle="Import/Export",
	hotkey=iup.K_E,
}
]====]

ui.tabs = iup.subsubtabtemplate {
	ui.presets.tab,
	--ui.export.tab, --TODO
	ui.options.tab,
}

--hack to do action on tab change
for n=1,2 do --n=tab number
	local old_action = ui.tabs[1][1][2][n][1][1].action
	ui.tabs[1][1][2][n][1][1].action = function(self, ...)
		local orig_tab = ui.tabs:GetTab()
		ui.LastDoubleClick.timestamp = nil --prevent false double-click
		old_action(self, ...)
		
		local new_tab = ui.tabs:GetTab()
		if new_tab ~= orig_tab then ui.LoadPresets(new_tab) end
	end
end

-- Create Main Dialog --

local frame = iup.vbox{
	iup.hbox{iup.fill{}, iup.label{title="Ship Loadouts Manager",font=Font.H3}, iup.fill{}},
	iup.hbox{iup.fill{}, iup.label{title="Version "..SLM.version}},
	alignment="ALEFT",
}

ui.dialog = iup.dialog{
	iup.vbox{
		iup.fill{},
		iup.hbox{
			iup.fill{},
			iup.pdarootframe{
			--iup.stationsubframe{
				iup.vbox{
					frame,
					iup.pdasubsubsubframe2 {ui.tabs},
					ui.element.close_button,
				},
				expand="NO",
				--gap=8,
			},
			iup.fill{},
		},
		iup.fill{},
	},
	BGCOLOR = "0 0 0 160 *",
	FULLSCREEN="YES",
	RESIZE="NO",
	BORDER="NO",
	MENUBOX="YES",
	TOPMOST="YES",
	DEFAULTESC=ui.element.close_button,
	DEFAULTENTER=ui.presets.element.filter_button,
}
ui.dialog:map()

--unused
function ui:Resize()
	--local label_height = tonumber(string.match(ui.presets.element.ship_label.size, 'x(%d+)'))
	
	--if label_height then
	--	local xscale = 202/22
	--end
	--console_print(label_height)
	--console_print(ui.presets.element.ship.size)
end

-- Display Dialog --

function ui:Show()
	SLM.data.LoadData()
	
	if not ui.dialog then ui:Init() end
	--ui.Resize(self) --TODO
	
	ui.LoadSettings()
	ui.LoadPresets(ui.presets.tab)
	ui.tabs.SetTab(self, 1) --make loadouts tab active
	
	ShowDialog(ui.dialog)
	--iup.Map(ui.dialog)
	iup.SetFocus(ui.presets.display.default_focus)
	iup.Refresh(ui.dialog)
end

function ui:Hide()
	--ui.tabs.value=0 --Necessary?
	HideDialog(ui.dialog)
end

function ui.element.close_button:action() ui:Hide() end

function ui.BEGIN_PURCHASE_SHIP_DIALOG() end

-- SHIP PURCHASE DIALOG --

ui.purchase = {}
ui.purchase.element = {}
ui.purchase.pane = {}

ui.purchase.element.ship_label = iup.label{title="Ship:"}
ui.purchase.element.ship_img = iup.label{title="", size="32x32", image="", visible="NO"}
ui.purchase.element.ship_list = iup.pdasubsubsublist{dropdown="YES", expand="HORIZONTAL", visible_items="10", action=ui.doListSearch}

ui.purchase.element.loadout_label = iup.label{title="Loadouts:"}
ui.purchase.element.loadout_list = iup.pdasubsubsublist{value=0, minsize="240x360", size="240x", expand="YES", action=ui.doList} --240x360
ui.purchase.element.buy_loadout = iup.stationbutton{title="Purchase Loadout", action=ui.doBuy, hotkey=iup.K_p,}
ui.purchase.element.loadout_price_label = iup.label{title="Price: "}
ui.purchase.element.loadout_price = iup.label{title="NOT AVAILABLE"}

ui.purchase.element.addons_label = iup.label{title="Addons:", expand="No"}
ui.purchase.element.addons_box = {}
ui.purchase.element.addons = {}
ui.purchase.element.addon_imgs = {}
for i=1,ui.MAX_NUM_PORTS do
	ui.purchase.element.addon_imgs[i] = iup.label{title="", size="32x32", image="", visible="NO"}
	ui.purchase.element.addons[i] = iup.label{title="none", size="400x", expand="HORIZONTAL"}
	
	ui.purchase.element.addons_box[i] = iup.hbox{
		ui.purchase.element.addon_imgs[i],
		iup.vbox{ iup.fill{}, ui.purchase.element.addons[i], iup.fill{} },
	}
end

ui.purchase.element.only_available_toggle = iup.stationtoggle{title="Show only available loadouts", action=ui.doOnlyAvailableToggle, key="IsOnlyAvailable", }

ui.purchase.element.help_button = iup.stationbutton{title="Help", hotkey=iup.K_F1,}
ui.purchase.element.cancel_button = iup.stationbutton{title="Cancel"}

ui.purchase.pane.loadout_info = iup.vbox{
	iup.fill{size=8},
	iup.hbox{
		iup.vbox{
			ui.purchase.element.addons_label,
			iup.vbox(ui.purchase.element.addons_box),
			iup.hbox{iup.fill{}},
		},
		iup.fill{size=2},
		iup.vbox{
			--TODO add color picker
			iup.fill{},
		},
	},
	iup.fill{size=8},
	iup.hbox{ui.purchase.element.loadout_price_label, ui.purchase.element.loadout_price},
	expand="NO",
}

ui.purchase.frame = iup.vbox{
	iup.hbox{iup.fill{}, iup.label{title="Purchase Ship",font=Font.H3}, iup.fill{}},
	iup.hbox{ui.purchase.element.help_button, iup.fill{}, },
	iup.fill{size=6},
	iup.hbox{
		iup.vbox{
			ui.purchase.element.loadout_label,
			ui.purchase.element.loadout_list,
		},
		iup.vbox{iup.fill{}},
		iup.vbox{
			ui.purchase.element.ship_label,
			iup.hbox{ iup.fill{}, ui.purchase.element.ship_img, ui.purchase.element.ship_list, iup.fill{} },
			iup.hbox{ui.purchase.element.only_available_toggle, iup.fill{}},
			iup.hbox{iup.fill{}},
			ui.purchase.pane.loadout_info,
		},
	},
	iup.fill{size=6},
	iup.hbox{ui.purchase.element.cancel_button, iup.fill{}, ui.purchase.element.buy_loadout},
}

ui.purchase.display = {
	list = ui.purchase.element.loadout_list,
	station_list = {},
	only_available = ui.purchase.element.only_available_toggle,
	filter_search = ui.purchase.element.ship_list,
	filter_type = "ship",
	
	loadout_name = {},
	ship = {name=ui.purchase.element.ship_list, img=ui.purchase.element.ship_img},
	addons = {names=ui.purchase.element.addons, imgs=ui.purchase.element.addon_imgs},
	price = ui.purchase.element.loadout_price,
	
	assign_presets = {{},{},{},{}},
	preset_labels = {{},{},{},{}},
	assign_defaults = {{},{},{},{}},
	default_labels = {{},{},{},{}},
	
	sector_name = {},
	station_name = {},
	
	buy_button=ui.purchase.element.buy_loadout,
	rename_button = {},
	delete_button = {},
	
	base_ship = true, --add BASE SHIP loadout with no addons to loadout list
	plain_search_override = true, --override pattern matching for compatibilty for ships with - in name
	
	default_focus = ui.purchase.element.loadout_list,
	
	dialog = ui.purchase.frame,
}
ui.purchase.element.loadout_list.elements = ui.purchase.display --non-standard attribute
ui.purchase.element.only_available_toggle.elements = ui.purchase.display --non-standard attribute
ui.purchase.element.ship_list.elements = ui.purchase.display --non-standard attribute
ui.purchase.element.buy_loadout.elements = ui.purchase.display --non-standard attribute
ui.purchase.frame.elements = ui.purchase.display --non-standard attribute

ui.purchase.dialog = iup.dialog{
	iup.vbox{
		iup.fill{},
		iup.hbox{
			iup.fill{},
			iup.stationhighopacityframe{
				iup.stationhighopacityframebg{
				--iup.pdarootframe{
				--iup.pdasubsubsubframe2{
					--iup.pdarootframebg{
					iup.vbox{ui.purchase.frame},
					expand="NO",
				}
			},
			iup.fill{},
		},
		iup.fill{},
	},
	BGCOLOR = "0 0 0 160 *",
	FULLSCREEN="YES",
	RESIZE="NO",
	BORDER="NO",
	MENUBOX="YES",
	TOPMOST="YES",
	DEFAULTESC=ui.purchase.element.cancel_button,
}
ui.purchase.dialog:map()

function ui.purchase:Show()
	SLM.data.LoadData()
	
	if not PlayerInStation() then return end --must be in station to show dialog
	
	if not ui.purchase.dialog then ui.purchase:Init() end
	--ui.Resize(self)
	
	--create list of ships sold by station
	local available_ships = {}
	for i=1,GetNumStationMerch() do
		local item = GetStationMerchInfo(i)
		
		if item.type=='ship' then
			table.insert(available_ships, item.name)
		end
	end
	
	table.sort(available_ships, sort_funcs.ship) --sort list by ship name
	
	--add ships to GUI loadout list
	for _,ship_name in ipairs(available_ships) do
		--available_ships[item.itemid] = true
		ui.PushListItem(ui.purchase.display.filter_search, ship_name) --add items from sorted list
	end
	
	local sel_name = filter_colorcodes(ui.GetListSelected(StationEquipmentBuyShipTab[1][3][2])):match('^%s*(.-)%s*%[') --extract name from selected ship in PDA
	ui.purchase.display.filter_search.value = ui.GetListItemIndex(ui.purchase.display.filter_search, sel_name) or 1 --set dropdown menu to ship selected from PDA
	
	ui.LoadPresets(ui.purchase.frame) --Generate list of loadouts
	
	ShowDialog(ui.purchase.dialog)
	iup.SetFocus(ui.purchase.display.default_focus)
	iup.Refresh(ui.purchase.dialog)
end

function ui.purchase:Hide()
	HideDialog(ui.purchase.dialog)
end

function ui.purchase:OnHelp()
	StationHelpDialog:Open(
[[This dialog allows you to select a loadout for a given ship and purchase it. You can change the ship from the dropdown menu. Only ships sold by the current station are listed.

Choose the BASE SHIP loadout if you just want a stock ship with no addons. Note that choosing this loadout is equivalent to the original function of the Purchase Ship button without this plug-in installed. The color of the BASE SHIP loadout is the color the ship will be (changeable from the color picker on the Ship -> Buy -> Buy Ship in the PDA).

Loadouts with addons that are not available at the current station will be prefaced with a *. You can hide these loadouts by enabling the "Show only available loadouts" checkbox.
]])
end
ui.purchase.element.help_button.action=ui.purchase.OnHelp

function ui.purchase.element.cancel_button:action() ui.purchase:Hide() end

-- Rename Loadout Dialog --

function ui.BEGIN_RENAME_DIALOG() end

function ui.MakeRenameDialog(ok_cb, isDefault, old_name)
	local rename_element = {}
	rename_element.input_text = iup.text{value="", size="400x", nc=20}
	rename_element.error_label = iup.label{title="", size="400", fgcolor="255 0 0"}
	rename_element.default_toggle = iup.stationtoggle{title="Save as universe-wide default", value="OFF"}
	rename_element.ok_button = iup.stationbutton{title="Save"}
	rename_element.cancel_button = iup.stationbutton{title="Cancel"}
	
	if isDefault~=nil then
		rename_element.default_toggle.value = isDefault and "ON" or "OFF"
		rename_element.default_toggle.active = "NO"
	end
	
	local rename_frame = iup.vbox{
		iup.hbox{iup.fill{}, iup.label{title=old_name and "Rename "..old_name or "Choose a name for the loadout",font=Font.H3}, iup.fill{}},
		rename_element.input_text,
		rename_element.error_label,
		rename_element.default_toggle,
		iup.hbox{rename_element.cancel_button, iup.fill{}, rename_element.ok_button},
		alignment="ALEFT",
	}
	
	local dlg = iup.dialog{
		iup.vbox{
			iup.fill{},
			iup.hbox{
				iup.fill{},
				iup.stationhighopacityframe{
					iup.stationhighopacityframebg{
					--iup.stationsubframe{
						rename_frame,
						expand="NO",
						gap=8,
					}
				},
				iup.fill{},
			},
			iup.fill{},
		},
		BGCOLOR = "0 0 0 160 *",
		FULLSCREEN="YES",
		RESIZE="NO",
		BORDER="NO",
		MENUBOX="YES",
		TOPMOST="YES",
		DEFAULTESC=rename_element.cancel_button,
		DEFAULTENTER=rename_element.ok_button,
	}
	dlg:map()
	
	function rename_element.default_toggle:action()
		if self.value=="ON" then
			rename_element.input_text.value = "DEFAULT"
			rename_element.input_text.active = "NO"
		else
			rename_element.input_text.value = ""
			rename_element.input_text.active = "YES"
		end
	end
	
	function rename_element.ok_button:action()
		local new_name = rename_element.input_text.value
		local isValid, error_msg = SLM.VerifyName(new_name)
		
		if rename_element.default_toggle.value=="OFF" and not isValid then
			rename_element.error_label.title=error_msg
			iup.Refresh(dlg)
			
			return
		end
		
		ok_cb(new_name)
		HideDialog(dlg)
	end

	function rename_element.cancel_button:action()
		HideDialog(dlg)
	end
	
	return dlg
end

--* END UI *--

function ui.LoadSettings()
	ui.options.element.backup_info.title = ""
	ui.options.element.restore_info.title = ""
	
	ui.options.element.use_color_toggle.value = SLM.vars.GetVars"IsUseColor" and "ON" or "OFF"
	ui.options.element.sort_presets_popup.value = SLM.vars.GetVars"SortPresetsBy"
	ui.options.element.enhanced_purchase_toggle.value = SLM.vars.GetVars"IsEnhancedPurchase" and "ON" or "OFF"
	ui.options.element.double_click_buy_toggle.value = SLM.vars.GetVars"IsDoubleClickBuy" and "ON" or "OFF"
	ui.options.element.case_insensitive_toggle.value = SLM.vars.GetVars"IsCaseInsensitive" and "ON" or "OFF"
	ui.options.element.pattern_matching_toggle.value = SLM.vars.GetVars"IsPatternMatching" and "ON" or "OFF"
	ui.options.element.gmt_toggle.value = SLM.vars.GetVars"IsGMT" and "ON" or "OFF"
end

--Returns true if loadout matches the given search critera
local function testSearch(loadout, name, search_criteria)
	local all_passed = true --assume all passed until proven otherwise
	
	for search_key, search_val in pairs(search_criteria) do
		local compare_to = type(filter_compare[search_key])=='function' and filter_compare[search_key](loadout, name) or {} --loadout data to search
		
		if type(search_val)~='boolean' then --ignore boolean values, not part of search (e.g. plain_search_override flag)
			for _,search_val_str in ipairs(search_val) do
				local found_match = false --assume false until match found
				
				for _,compare_to_str in ipairs(compare_to) do
					if search_val_str=='' or string.find(compare_to_str, search_val_str, 1, not search_criteria.plain_search_override and not SLM.vars.GetVars"IsPatternMatching") then --match search string
						found_match = true; break
					end
				end
				
				if not found_match then all_passed = false; break end
			end
			
			if not all_passed then break end
		end
	end
	
	return all_passed
end

--Populates loadout list
function ui:LoadPresets()
	local display = self.elements --or ui.presets.display
	if not display then --[[console_print'ERROR! element display not found!';]] return end --display not defined for options tab
	
	display.list[1] = nil --clear loadout list
	
	local is_in_station = PlayerInStation() and not GetStationType() --not in capship
	local is_only_available = SLM.vars.GetVars"IsOnlyAvailable" and is_in_station
	display.only_available.value = is_only_available and "ON" or "OFF"
	display.only_available.active = is_in_station and "ON" or "OFF"
	
	--get search string
	local search = display.filter_search
	local search_type = iup.GetType(search)
	local search_str = search_type=='list' and search[search.value] or (search_type=='text' and search.value or '') --extract search string
	if SLM.vars.GetVars"IsCaseInsensitive" then search_str = search_str:lower() end
	
	--get search type
	local search_target = display.filter_type
	local search_target_type = type(search_target)=='userdata' and iup.GetType(search_target) or type(search_target)
	local search_target_str = (search_target_type=='list' and search_target[search_target.value] or (search_target_type=='text' and search_target.value or search_target)):lower() --string for search type
	
	
	--reformat search data into table
	local search_criteria = {}
	if search_target_str == "advanced" then
		local adv_search = Choco.util.str2tbl(search_str, "%s+") --split search entries at spaces
		
    	for _,v in ipairs(adv_search) do
    		local key, val = v:match'(%w+):(.+)' --extract key and value, separated by colon
    		if key and val then
    			val = Choco.util.str2tbl(val, ",%s*") --split at commas
    			search_criteria[key] = val
    		else end --TODO error on invalid adv search string
    	end
    else search_criteria[search_target_str] = Choco.util.str2tbl(search_str:gsub("%s+",''), ",%s*") end --remove whitespace and split at commas
    
    search_criteria.plain_search_override = self.elements.plain_search_override --true to override pattern matching var and use plain search
    if self.elements.plain_search_override then
    	for i,str in ipairs(search_criteria[search_target_str] or {}) do
    		search_criteria[search_target_str][i] = "^"..str:gsub("%-",'%%-').."$"
    	end
    end
    
    --create sorted list of preset names
	local sorted_order = {} --sorted list of all loadouts
	for name,loadout in pairs(SLM.data.GetLoadouts()) do
		if testSearch(loadout, name, search_criteria) then
			local price, is_avail = SLM.QuickAvailability(loadout)
			
			if not is_only_available or is_avail then
				--add loadout to list
				local loadout_entry = {
					loadout = loadout,
					name = name,
					name_lower = name:lower(), --so lower() is not called multiple times in sort_funcs
					ship_name = InvManager.GetInventoryItemParameter(loadout.ship, 'Name'):lower(),
					color = SLM.GetLoadoutColor(loadout) or ui.COLOR.WHITE,
					price = price, --may be nil, calc later if needed
					avail = is_avail,
				}
				table.insert(sorted_order, loadout_entry)
			end
		end
	end
	
	table.sort(sorted_order, sort_funcs[ ui.options.sort_types[SLM.vars.GetVars"SortPresetsBy"] ])
	
	-- Populate preset list contents --
	
	--entry for stock ship with no addons
	if display.base_ship then ui.PushListItem(display.list, rgbtohex(ShipPalette_string[GetShipPurchaseColor()+1]).."BASE SHIP") end --index shift: ShipPallette is 1-256, not 0-255
	
	--add active ship to top of list
	local special_loadout = "ACTIVE SHIP"
	if HasActiveShip() then --only include ACTIVE SHIP if player has an active ship
		local temp_load = ui.GetLoadoutByPreset(special_loadout)
		if testSearch(temp_load, special_loadout, search_criteria) then
			ui.PushListItem(display.list, ui.COLOR.WHITE..special_loadout)
		end
	end
	
	for _,entry in ipairs(sorted_order) do
		local full_name --with * appended for non-available loadouts
		if not entry.avail then
			full_name = entry.color..'*'..entry.name
		else full_name = entry.color..entry.name end
		
		ui.PushListItem(display.list, full_name) --add items from sorted list
	end
	
	
	for i=1,4 do
		special_loadout = "DEFAULT "..i
		if ShipPresets[i] and testSearch(ui.GetLoadoutByPreset(special_loadout), special_loadout, search_criteria) then --only if defined and matches search
			ui.PushListItem(display.list, ui.COLOR.WHITE..special_loadout) --add default presets to end of list
		end
		
		--assign text to default preset buttons
		local default_loadout = ShipPresets[i]
		if default_loadout then
			display.default_labels[i].title = ui.COLOR.GREY..SLM.GetTruncatedShipName(default_loadout)
		else display.default_labels[i].title = ui.COLOR.GREY.."UNASSIGNED" end
	end
	
	if ui.GetListSize(display.list)==0 then ui.PushListItem(display.list, ui.COLOR.WHITE.."-NO LOADOUTS FOUND-") end --add indication for none found
	
	--populate station text
	if PlayerInStation() and not GetStationType() then --not in capship
		local stationid = GetStationLocation()
		local station_presets = SLM.data.GetLoadoutByStation(stationid) or {}
		
		display.sector_name.title = ShortLocationStr(GetCurrentSectorid())..": "
		display.station_name.title = GetStationName(stationid)
		
		for i=1,4 do
			if station_presets[i] then
				local color = SLM.GetLoadoutColor(SLM.data.GetLoadoutByName(station_presets[i])) or ''
				display.preset_labels[i].title = color..station_presets[i]
			else display.preset_labels[i].title = "UNASSIGNED" end
			
			display.assign_presets[i].active = "YES" --enable preset buttons
		end
	else
		display.sector_name.title = ShortLocationStr(GetCurrentSectorid())..": "
		display.station_name.title = "NOT DOCKED AT STATION"
		
		for i=1,4 do
			display.preset_labels[i].title = ""
			display.assign_presets[i].active = "NO"
		end
	end
	
	display.list.value = 1 --select first item
	ui.LastDoubleClick.timestamp = nil --prevent false double-click
	ui.UpdateListSelection(display.list) --update loadout info panel
	
	return sorted_order
end
ui.presets.element.filter_button.action=ui.LoadPresets
ui.presets.element.showOnlyAvailable.action=function(self)
	ui.varsToggle.action(self)
	ui.LoadPresets(self)
end

--Strips out color codes and removes leading *
function ui.FilterPresetName(name) return filter_colorcodes(name):match"^%**(.*)" end --TODO local?

--loadout for stock ship with no addons -- Ship must be sold at current station
function ui.CreateBaseLoadout(ship_name)
	local loadout = {
		ports={0},
		groups={ {[2]=true},{},{},{},{},{},{},{},{},{},{},{},{},{},{},{},{},{} },
	}
	
	--find ship
	if not PlayerInStation() then return false end --must be docked in station
	for i=1,GetNumStationMerch() do
		local item = GetStationMerchInfo(i)
		
		if item.type=='ship' and item.name==ship_name then loadout.ship=item.itemid; break end
	end
	if not loadout.ship then return end --return nil if ship not sold at station
	
	loadout.shipcolor=GetShipPurchaseColor() --selected ship color from picker
	
	return loadout
end

--Returns loadout (table) that has the specified name
function ui.GetLoadoutByPreset(name)
	if not next(SLM.data) then SLM.LoadData() end
	
	if type(name)~='string' then return nil end
	name = ui.FilterPresetName(name) --strip color codes and remove leading *
	
	--return loadout for saved preset
	local loadout = SLM.data.GetLoadoutByName(name)
	if loadout~=nil then return loadout end
	
	--return default loadout: name="DEFAULT N"
	local default_num = tonumber(name:match'^DEFAULT (%d+)$')
	if default_num and default_num>=1 and default_num<=4 then return ShipPresets[default_num] end
	
	--return loadout of active ship: name="ACTIVE SHIP"
	if name=="ACTIVE SHIP" then return SaveShipLoadout(GetActiveShipID()) end
	
	--return dummy loadout for BASE SHIP
	local ship_name = name:match"^BASE SHIP: (.+)"
	if ship_name then return ui.CreateBaseLoadout(ship_name) end
	
	return nil --invalid name specified
end

-- List Functions --

function ui.GetListSelected(list)
	local num = tostring(list.vo_listsel)
	
	--local num = tostring(list.value)
	local val = list[num]
	
	return val
end

function ui.GetListSize(list)
	local size = 0
	while list[size + 1] do size=size+1 end
	return size
end

--Adds an item to the end of a iup list
function ui.PushListItem(list, item)
	list.value = list.value or 0
	
	local new_val = list.value+1
	list[tostring(new_val)] = item
	--list[tostring(list.value)] = item
	list.value = new_val
	
	--list.value = list.value or 0
	--list['insertitem'..list.value+1] = item
	
	--wrong:
	--local index = ui.GetListSize(list)+1
	--list[index] = item
	--list.value = index
end

--Removes the selected item from a list, returns the value of the item removed
function ui.RemoveListItem(list, sel)
	local size = ui.GetListSize(list)
	sel = sel or list.vo_listsel or size
	
	if sel~=0 then
		local item = list[tostring(sel)]
		
		while tonumber(sel) < size do
			list[sel] = list[sel+1]
			sel = sel+1
		end
		
		list[tostring(size)] = nil
		list.value = size-1
		list.vo_listsel = sel-1
		
		return item
	end
end

function ui.GetListItemIndex(list, item)
	for i=1,ui.GetListSize(list) do
		local val = list[i]
		if item==val or item==filter_colorcodes(val) then return i end
	end
	
	return nil
end

return ui